@@ -3,7 +3,7 @@
 ;; Copyright (C) 1996,1997 Free Software Foundation, Inc.
 
 ;; Author: MORIOKA Tomohiko <morioka@jaist.ac.jp>
-;; Version: $Id: file-detect.el,v 1.2 1999/10/29 04:41:42 iwa Exp $
+;; Version: $Id: file-detect.el,v 1.8 2001/02/02 00:31:14 yutopia Exp $
 ;; Keywords: file detection, install, module
 ;; Status: obsoleted
 
@@ -33,6 +33,7 @@
 
 (require 'path-util)
 
-(provide 'file-detect)
+(require 'product)
+(product-provide (provide 'file-detect) (require 'apel-ver))
 
 ;;; file-detect.el ends here
