@@ -5,8 +5,8 @@
 ;;;
 ;;; Author: Toshihiko Ueki <toshi@he.kobelcosys.co.jp>
 ;;; Created:       1997/04/20
-;;; Last modified: 1998/09/06
-;;; Version: $Revision: 1.2 $
+;;; Last modified: 2000/06/14
+;;; Version: $Revision: 1.5 $
 
 ;;; This file is not part of mailcrypt.
 ;;;
@@ -40,7 +40,7 @@
 ;;; @ version definition
 ;;;
 
-(defconst cmail-crypt-version "$Revision: 1.2 $" "*cmail-crypt version")
+(defconst cmail-crypt-version "$Revision: 1.5 $" "*cmail-crypt version")
 
 ;;; @ variables
 ;;;
@@ -48,8 +48,12 @@
 (defvar cmail-crypt-automatic-decrypt/verify t
   "*If non-nil then decrypt message and verify signature automatically.")
 
-(if (not (string= mc-version "3.4"))
-    (defvar cmail-crypt-default-version "5.0" "*Default PGP program version"))
+(if (string= mc-version "3.4")
+    nil
+  (defvar cmail-crypt-default-version "5.0"
+    "*Default PGP program version")
+  (defvar cmail-crypt-altpgp-version "5.0"
+    "*PGP version to use instead of GnuPG. Must specify \"2.6\" or \"5.0\"."))
 
 ;;; @ working variables
 ;;;
@@ -71,7 +75,11 @@
 
 (defconst mc-pgp-trademark-string "^Pretty Good Privacy(tm)"
   "*Text for trademark string of PGP.")
-(defconst mc-pgp-signature-begin-line "^-----BEGIN PGP SIGNATURE-----"
+(defconst mc-gpg-trademark-string "^GNU Privacy Guard"
+  "*Text for trademark string of GPG.")
+(defconst mc-pgp-sig-begin-line "^-----BEGIN PGP \\(SIGNATURE\\|MESSAGE\\)-----"
+  "*Text for start of PGP signature.")
+(defconst mc-pgp-signature-begin-line (concat mc-pgp-sig-begin-line "\r?$")
   "*Text for start of PGP signature.")
 (defconst mc-pgp-verify-error "ERROR: Temporary process error occurred.\n"
   "*Text for temporary error. Maybe pgp process error occurred.")
@@ -83,8 +91,10 @@
   "*Regular expression matching an error from PGP")
 (defconst mc-pgp-ukid-re "^Signature by unknown keyid: 0x.*\n"
   "*Regular expression matching an unknown PGP keyid")
-(defconst cmail-crypt-verified-header "X-PGP-Verify: "
+(defconst cmail-crypt-pgp-verified-header "X-PGP-Verify: "
   "*Header field for PGP.")
+(defconst cmail-crypt-gpg-verified-header "X-GnuPG-"
+  "*Header field for GnuPG.")
 
 ;;; @ PGP program I/O code definition
 ;;;
@@ -107,17 +117,27 @@
   (defun cmail-crypt-change-default-version ()
     "*Toggle using pgp version."
     (interactive)
-    (if (string= cmail-crypt-default-version "5.0")
-	(setq cmail-crypt-default-version "2.6")
-      (setq cmail-crypt-default-version "5.0"))
+    (cond
+     ((string= cmail-crypt-default-version "5.0")
+      (setq cmail-crypt-default-version "2.6"))
+     ((string= cmail-crypt-default-version "2.6")
+      (setq cmail-crypt-default-version "GPG"))
+     (t
+      (setq cmail-crypt-default-version "5.0")))
     (cmail-crypt-set-pgp-default-version))
 
-  (defun cmail-crypt-change-pgp-version-temporarily ()
-    "*Toggle using pgp version temporarily."
+  (defun cmail-crypt-change-pgp-version-temporarily (&optional ver)
+    "*Change using pgp version temporarily."
     (interactive)
-    (if (string= cmail-crypt-temporary-version "5.0")
-	(setq cmail-crypt-temporary-version "2.6")
-      (setq cmail-crypt-temporary-version "5.0"))
+    (if ver
+	(setq cmail-crypt-temporary-version ver)
+      (cond
+       ((string= cmail-crypt-temporary-version "5.0")
+	(setq cmail-crypt-temporary-version "2.6"))
+       ((string= cmail-crypt-temporary-version "2.6")
+	(setq cmail-crypt-temporary-version "GPG"))
+       (t
+	(setq cmail-crypt-temporary-version "5.0"))))
     (cmail-crypt-set-pgp-version))
 
   (defun cmail-crypt-set-pgp-default-version ()
@@ -128,25 +148,18 @@
 
   (defun cmail-crypt-set-pgp-version ()
     "*Set pgp version."
-    (if (string= cmail-crypt-temporary-version "5.0")
-	(mc-setversion "5.0")
+    (cond
+     ((string= cmail-crypt-temporary-version "GPG")
+      (mc-setversion "gpg"))
+     ((string= cmail-crypt-temporary-version "5.0")
+      (mc-setversion "5.0"))
+     (t
       (mc-setversion "2.6")
-      (cmail-crypt-26io-code-definition)))
+      (cmail-crypt-26io-code-definition))))
 
   (defun cmail-crypt-toggle-mc-write-mode-menu-bar ()
-    (if (string= cmail-crypt-temporary-version "5.0")
-	(easy-menu-define mc-write-mode-menu (if mc-xemacs-p nil (list mc-write-mode-map))
-	  "Mailcrypt write mode menu."
-	  '("Mailcrypt"
-	    ["Encrypt Message" mc-encrypt t]
-	    ["Sign Message" mc-sign t]
-	    ["Insert Public Key" mc-insert-public-key t]
-	    ["Fetch Key" mc-pgp-fetch-key t]
-	    ["Encrypt for Remailer(s)" mc-remailer-encrypt-for-chain t]
-	    ["Insert Pseudonym" mc-remailer-insert-pseudonym t]
-	    ["Insert Response Block" mc-remailer-insert-response-block t]
-	    ["Forget Passphrase(s)" mc-deactivate-passwd t]
-	    ["Change PGP version -> 2.6" cmail-crypt-change-pgp-version-temporarily t]))
+    (cond
+     ((string= cmail-crypt-temporary-version "5.0")
       (easy-menu-define mc-write-mode-menu (if mc-xemacs-p nil (list mc-write-mode-map))
 	"Mailcrypt write mode menu."
 	'("Mailcrypt"
@@ -158,7 +171,33 @@
 	  ["Insert Pseudonym" mc-remailer-insert-pseudonym t]
 	  ["Insert Response Block" mc-remailer-insert-response-block t]
 	  ["Forget Passphrase(s)" mc-deactivate-passwd t]
-	  ["Change PGP version -> 5.0" cmail-crypt-change-pgp-version-temporarily t])))))
+	  ["Change PGP version -> 2.6" cmail-crypt-change-pgp-version-temporarily t])))
+     ((string= cmail-crypt-temporary-version "2.6")
+      (easy-menu-define mc-write-mode-menu (if mc-xemacs-p nil (list mc-write-mode-map))
+	"Mailcrypt write mode menu."
+	'("Mailcrypt"
+	  ["Encrypt Message" mc-encrypt t]
+	  ["Sign Message" mc-sign t]
+	  ["Insert Public Key" mc-insert-public-key t]
+	  ["Fetch Key" mc-pgp-fetch-key t]
+	  ["Encrypt for Remailer(s)" mc-remailer-encrypt-for-chain t]
+	  ["Insert Pseudonym" mc-remailer-insert-pseudonym t]
+	  ["Insert Response Block" mc-remailer-insert-response-block t]
+	  ["Forget Passphrase(s)" mc-deactivate-passwd t]
+	  ["Change PGP version -> GPG" cmail-crypt-change-pgp-version-temporarily t])))
+     (t
+      (easy-menu-define mc-write-mode-menu (if mc-xemacs-p nil (list mc-write-mode-map))
+	"Mailcrypt write mode menu."
+	'("Mailcrypt"
+	  ["Encrypt Message" mc-encrypt t]
+	  ["Sign Message" mc-sign t]
+	  ["Insert Public Key" mc-insert-public-key t]
+	  ["Fetch Key" mc-pgp-fetch-key t]
+	  ["Encrypt for Remailer(s)" mc-remailer-encrypt-for-chain t]
+	  ["Insert Pseudonym" mc-remailer-insert-pseudonym t]
+	  ["Insert Response Block" mc-remailer-insert-response-block t]
+	  ["Forget Passphrase(s)" mc-deactivate-passwd t]
+	  ["Change PGP version -> 5.0" cmail-crypt-change-pgp-version-temporarily t]))))))
 
 ;;; @ PGP decoder
 ;;;
@@ -172,10 +211,9 @@
 	(setq cmail-crypt-perform-verify t)
 	(setq cmail-crypt-perform-decrypt t))
     (setq cmail-crypt-perform-verify nil)
-
-(if (not (string= mc-version "3.4"))
-    (defvar cmail-crypt-temporary-version cmail-crypt-default-version))
     (setq cmail-crypt-perform-decrypt nil))
+  (if (not (string= mc-version "3.4"))
+      (setq cmail-crypt-temporary-version cmail-crypt-default-version))
   (cmail-show-contents (cmail-get-page-number-from-summary)))
 
 (defun cmail-crypt-decrypt/verify ()
@@ -200,17 +238,38 @@
 (defun cmail-crypt-check-decrypt/verify (beg end func)
   "*Check if this message has signature or encrypted message."
   (save-excursion
-    (if (re-search-forward (concat "^" beg) nil t)
-	(if (re-search-forward (concat "^" end) nil t)
-	    (let ((buffer-read-only nil))
+    (if (cmail-crypt-check-decrypt/verify-search-forward beg)
+	(if (cmail-crypt-check-decrypt/verify-search-forward end)
+	    (let ((buffer-read-only nil)
+		  (gpgver (concat mc-pgp-sig-begin-line "\nVersion: GnuPG"))
+		  tmpver cmail-crypt-temporary-version)
 	      (beginning-of-buffer)
+	      (if (null (string= mc-version "3.4"))
+		  (progn
+		    (if (null (re-search-forward gpgver nil t))
+			(if (string= cmail-crypt-default-version "GPG")
+			    (setq tmpver cmail-crypt-altpgp-version)
+			  (setq tmpver cmail-crypt-default-version))
+		      (setq tmpver "GPG")
+		      (beginning-of-buffer))
+		    (save-excursion
+		      (cmail-crypt-change-pgp-version-temporarily tmpver))))
 	      (defalias 'tmp-err 'error)
 	      (defalias 'error 'message)
-	      (funcall func)
-	      (defalias 'error 'tmp-err)
+	      (unwind-protect
+		  (funcall func)
+		(defalias 'error 'tmp-err))
 	      (set-buffer-modified-p nil)
 	      (cmail-go-summary))))))
 
+(defun cmail-crypt-check-decrypt/verify-search-forward (s)
+  "*Helper function for cmail-crypt-check-decrypt/verify
+and cmail-crypt-after-verify."
+  (let (b e)
+    (setq b (if (string-match "^\\^" s) "" "^"))
+    (setq e (if (string-match "\r?\\$$" s) "" "\r?$"))
+    (re-search-forward (concat b s e) nil t)))
+
 (defun cmail-crypt-check-mail-buffer ()
   "*Check if \"*cmail-mail-buffer\" has the contents of this page."
   (let ((cmail-crypt-perform-verify nil) (cmail-crypt-perform-decrypt nil))
@@ -242,7 +301,8 @@
 (defun cmail-crypt-decrypt-func ()
   "*Decrypt the contents."
   (if (not (car (mc-decrypt-message)))
-      (cmail-select-buffer *cmail-mail-buffer)))
+      (cmail-select-buffer *cmail-mail-buffer))
+  (cmail-crypt-after-decrypt/verify nil))
 
 (defun cmail-crypt-verify-signature ()
   "*Verify the signature in the current message."
@@ -261,38 +321,49 @@
 (defun cmail-crypt-verify-func ()
   "*Verify the signature."
   (mc-verify-signature)
-  (cmail-crypt-after-verify))
+  (cmail-crypt-after-decrypt/verify t))
 
-(defun cmail-crypt-after-verify ()
+(defun cmail-crypt-after-decrypt/verify (after-verify)
   "*Display verified message."
   (re-search-forward "\n\n")
   (backward-char 1)
-  (insert-string cmail-crypt-verified-header)
   (save-excursion
-    (let ((region (cmail-crypt-get-message)))
-      (if (null region)
-	  (insert-string mc-pgp-verify-error)
-        (cmail-insert-buffer-substring mc-buffer-name (car region) (cdr region))
-	(insert-string "\n")))
-    (save-restriction
-      (let (beg end)
-	(save-excursion
-	  (if (re-search-forward (concat "^" mc-pgp-signed-begin-line "\n") nil t)
+    (if (string= cmail-crypt-temporary-version "GPG")
+	nil
+      (let ((region (cmail-crypt-get-message)))
+	(if (null region)
+	    nil
+	  (insert-string cmail-crypt-pgp-verified-header)
+	  (cmail-insert-buffer-substring mc-buffer-name (car region) (cdr region))
+	  (insert-string "\n")))))
+  (if after-verify (cmail-crypt-after-verify)))
+
+(defun cmail-crypt-after-verify ()
+  "*Display verified message."
+  (let ((beg (point)) end)
+    (if (cmail-crypt-check-decrypt/verify-search-forward mc-pgp-signed-begin-line)
+	(progn
+	  (replace-match "")
+	  (if (string= cmail-crypt-temporary-version "GPG")
 	      (progn
-		(setq beg (match-beginning 0))
-		(replace-match ""))
-	    (setq beg (point)))
-	  (if (re-search-forward (concat "^" mc-pgp-signed-end-line "\n") nil t)
-	      (setq end (match-end 0))
-	    (setq end (point-max)))
-	  (narrow-to-region beg end))
-	(if (re-search-forward mc-pgp-signature-begin-line nil t)
-	    (delete-region (match-beginning 0) (point-max)))
-	(delete-blank-lines)
-	(goto-char beg)
-	(delete-blank-lines)
-	(while (re-search-forward "^- -" nil t)
-	  (replace-match "-"))))))
+		(forward-char 1)
+		(delete-region beg (point))
+		(insert-string cmail-crypt-gpg-verified-header)
+		(end-of-line)
+		(insert-string "\n")))
+    (setq beg (point)))
+    (if (cmail-crypt-check-decrypt/verify-search-forward mc-pgp-signed-end-line)
+	(setq end (match-end 0))
+      (setq end (point-max)))
+    (narrow-to-region beg end))
+  (if (re-search-forward mc-pgp-signature-begin-line nil t)
+      (delete-region (match-beginning 0) (point-max)))
+  (delete-blank-lines)
+  (goto-char beg)
+  (widen)
+  (delete-blank-lines)
+  (while (re-search-forward "^- -" nil t)
+    (replace-match "-"))))
 
 (defun cmail-crypt-get-message ()
   "*Get verification result in \"mc-buffer-name\" buffer."
