@@ -1319,7 +1319,8 @@
   (let ((gbw (get-buffer-window *cmail-summary-buffer)))
     (if gbw
 	(select-window gbw))
-    (cmail-mail2 t)))
+    (cmail-mail2 t nil nil nil nil nil nil nil
+		 (cmail-cc-me-p 'when-new-compose))))
 
 (defun cmail-folders-update (&optional rescan)
   "$BA4$F$N%U%)%k%@$r%;!<%V$7$F%U%)%k%@0lMw2hLL$r99?7$9$k(B.
