@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-misc.el - utility functions for cmail
 ;;;
-;;;  $Author: iwa $
+;;;  $Author: MaNakagawa $
 ;;;  created at: Mon Jul 12 10:44:23 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto
@@ -61,11 +61,12 @@
       (setq end (point))
       (narrow-to-region beg end))))
 
+(defvar *cmail-re-head-bdr "\n\n")
 (defun cmail-head-max ()
   "$B%+%l%s%H%Z!<%8$N%X%C%@$N:GBg%]%$%s%H(B($B%\!<%@9T$N;O$a(B)$B$rJV$9(B."
   (save-excursion
     (let ((max (cmail-page-max)))
-    (if (not (search-forward "\n\n" max 1))
+    (if (not (re-search-forward *cmail-re-head-bdr max 1))
 	max
       (forward-line -1)
       (point)))))
@@ -187,6 +188,16 @@
 	(if (null sep) (setq found t)))
       str)))
 
+(defun cmail-get-field-values-in-mail-buffer
+  (field-regexp &optional sep sep2 header page)
+  "FIELD-REGEXP$B$KEv$F$O$^$k%X%C%@$N%U%#!<%k%ICM$rJV$9!#%a!<%k%P%C%U%!MQ!#(B
+$B$b$H$N$O%U%)%k%@%P%C%U%!MQ!#(B"
+  (let ((*cmail-re-head-bdr (if header header
+			      (concat "^" mail-header-separator "$")))
+	(*cmail-re-bdr (if page page *cmail-re-bdr)))
+    (cmail-get-field-values field-regexp sep sep2)))
+(defalias 'cmail-get-field-values2 'cmail-get-field-values-in-mail-buffer)
+
 (defun cmail-trans-date-form (date)
   "'Tue, 30 Jun 92 ...'$B$^$?$O(B, '3 Mar 92 ...'$B$H$$$C$?(BDATE$B$NF|IU$+$i(B,
 '92/06/30'$B$H$$$&7A$KJQ49$9$k(B."
