@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-reply.el - Mailing, forwarding, and replying functions
 ;;;
-;;;  $Author: iwa $
+;;;  $Author: MaNakagawa $
 ;;;  created at: Mon Jul 12 11:01:47 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto
@@ -25,10 +25,10 @@
 
 (provide 'cmail-reply)
 (require 'sendmail)
-(require 'rfc822)
-(eval-when-compile (require 'cl))
+;(require 'rfc822)
 (require 'mail-utils)
-(require 'cmail)
+(eval-when-compile (require 'cl))
+;(require 'cmail)
 
 ;; !!!! $B%a%$%k$r=P$9(B !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 (defun cmail-mail (&optional user subject reply cc replybuffer action
@@ -77,7 +77,10 @@
 		   (cmail-format-resource1 'mail2-3 bufname))))
 	  ((eq query ?i)		; include
 	   (setq noerase t)
-	   (setq last-message (cmail-get-resource 'mail2-4)))
+	   (setq last-message (cmail-get-resource 'mail2-4))
+	   ;; when noerase replybuffer will not be copied, and as a result
+	   ;; mail-yank-original will not be called.
+	   (setq mail-reply-action replybuffer))
 	  (t
 	   (set-buffer-modified-p nil)
 	   (setq last-message (cmail-get-resource 'mail2-5)))))
@@ -117,30 +120,28 @@
     (setq actions mail-send-actions)
 
     (flet ((mail-mode () (cmail-mail-mode)))
+      (if (eq reply 'forward)
+	  ;; 'forward is set in cmail-forward for query ?i
+	  (setq reply nil))
       (mail noerase user subject reply cc replybuffer nil))
 
     (setq mail-send-actions (append actions action))
     (if cmail-default-from
 	(save-excursion
 	  (goto-char (point-min))
-	  (if (cmail-get-field-values "From")
+	  (if (cmail-get-field-values-in-mail-buffer "From")
 	      nil
 	    (insert "From: " cmail-default-from "\n"))))
     (setq mail-reply-buffer replybuffer)
     (message last-message)
     (cmail-mail-overload-functions)
+    (make-local-variable 'truncate-partial-width-windows)
+    (setq truncate-partial-width-windows nil)
     (run-hooks 'cmail-mail-hook)))
 
 (defun cmail-mail-overload-functions ()
   (setq mode-line-buffer-identification
 	"cmail: *mail*		"))
-;;  (if cmail-use-X-Nsubject
-;;	(local-set-key "\C-c\C-f\C-n" 'cmail-mail-x-nsubject))
-;;  (local-set-key "\C-c\C-y" 'cmail-yank-original)
-;;  (local-set-key "\C-c\C-s" 'cmail-send)
-;;  (local-set-key "\C-c\C-c" 'cmail-send-and-exit)
-;;  (local-set-key "\C-i" 'cmail-complete-address-or-tab)
-;;  (local-set-key "\C-c\C-r" 'cmail-caesar-region))
 
 (defun cmail-mail-from-summary (&optional user subj)
   "$B%5%^%jCf$+$i%a%$%k$r=P$9(B."
@@ -238,14 +239,14 @@
       (copy-to-buffer *cmail-user-mail-agent-buffer (point) (cmail-head-max))
       (set-buffer *cmail-user-mail-agent-buffer))
 
-    (setq dat (cmail-get-field-values "Date"))
-    (setq sbj (cmail-get-field-values "Subject"))
+    (setq dat (cmail-get-field-values-in-mail-buffer "Date"))
+    (setq sbj (cmail-get-field-values-in-mail-buffer "Subject"))
     (if cmail-use-X-Nsubject
-	(setq xnsbj (cmail-get-field-values "X-Nsubject")))
+	(setq xnsbj (cmail-get-field-values-in-mail-buffer "X-Nsubject")))
 
-    (setq ref (cmail-get-field-values "References"))
-    (setq mid (cmail-get-field-values "Message-Id"))
-    (setq from (cmail-get-field-values "From"))
+    (setq ref (cmail-get-field-values-in-mail-buffer "References"))
+    (setq mid (cmail-get-field-values-in-mail-buffer "Message-Id"))
+    (setq from (cmail-get-field-values-in-mail-buffer "From"))
     (if (and (not nocmail) cmail-mail-callback)
 	(setq action (list (cons 'cmail-reply-callback
 				 (list cmail-current-folder page mid)))))
@@ -274,23 +275,27 @@
 	    (mapcar (function (lambda (addr) (list addr))) dont-cc-list))
 
       ;; $B%"%I%l%9J8;zNs$r%j%9%H$K$7$F$+$i%k!<%kE,MQ(B
-      (setq frm (cmail-get-field-values "From" ", ")
-	    rplto (cmail-get-field-values "Reply-To" ", "))
+      (setq frm (cmail-get-field-values-in-mail-buffer "From" ", ")
+	    rplto (cmail-get-field-values-in-mail-buffer "Reply-To" ", "))
       (if (or to-all
 	      (cmail-address-match self-addr-regexp (cmail-address-list frm)))
 	  (progn
 	    (setq usr (cmail-address-list (or rplto frm)))
 	    (setq usr (cmail-dont-reply-to usr self-list))
 	    (setq usr (append usr (cmail-address-list
-				   (cmail-get-field-values "To" ", "))))
+				   (cmail-get-field-values-in-mail-buffer "To" ", "))))
 	    (setq usr (cmail-unify-dup-addresses usr))
 	    (setq ccc (cmail-address-list
-		       (cmail-get-field-values "CC" ", ")))
+		       (cmail-get-field-values-in-mail-buffer "CC" ", ")))
 	    (if (and ignore-reply-to-p rplto)
 		(setq ccc (append (cmail-address-list frm) ccc)))
 	    (setq ccc (cmail-unify-dup-addresses ccc))
 	    (setq ccc (cmail-dont-reply-to ccc usr))
 	    (setq usr (cmail-dont-reply-to usr dont-reply-to-list))
+	    (if (null usr)
+		(if (and (null ignore-reply-to-p) rplto)
+		    (setq usr (cmail-address-list rplto))
+		  (setq usr (cmail-address-list frm))))
 	    (setq ccc (cmail-dont-reply-to ccc dont-cc-list)))
 	(if ignore-reply-to-p
 	    (setq usr (cmail-address-list frm))
@@ -347,12 +352,11 @@
 		   (if to-all (cmail-cc-me-p 'when-follow-up)
 		     (cmail-cc-me-p 'when-reply))))
     (set-buffer "*mail*")
-    (make-local-variable 'truncate-partial-width-windows)
-    (setq truncate-partial-width-windows nil)
-    (if (not yankorigin)
-	nil
-      (set-buffer "*mail*")
-      (cmail-yank-original nil))
+    (if yankorigin
+	(progn
+	  (cmail-yank-original nil)
+	  (if (and (featurep 'cmail-use-multi-highlight) cmail-use-multi-highlight)
+	      (cmail-modify-keywords-highlight-citation t))))
     (save-excursion
       (if xnsbj
 	  (progn
@@ -361,7 +365,7 @@
       (if (and cmail-make-References-header mid)
 	  (let (oref)
 	    (goto-char (point-min))
-	    (setq oref (cmail-get-field-values "References"))
+	    (setq oref (cmail-get-field-values-in-mail-buffer "References"))
 	    (setq ref (cmail-remove-duplicate-message-id
 		       (if ref (concat ref " " mid) mid) oref))
 	    (if (string= ref "")
@@ -577,7 +581,7 @@ address string extracted."
     (if (string= *cmail-unknown-field-value sbj)
 	(cmail-get-resource 'make-reply-headings-1) sbj))
    ((= ctl ?s)
-    (substring sbj (string-match "\\(Re: *\\)*" sbj)))
+    (substring sbj (string-match "\\(Re:\\s *\\)*" sbj)))
    ((= ctl ?d) date)
    ((= ctl ?D) (cmail-trans-date-form date))
    ((= ctl ?n) name)
@@ -603,7 +607,7 @@ address string extracted."
 
 (defun cmail-forward (&optional user)
   "$B%+%l%s%H5-;v$r%U%)%o!<%I$9$k(B."
-  (interactive (list (cmail-complete-field-to)))
+  (interactive)
   (let ((page (cmail-get-page-number-from-summary))
 	from subject)
     (beginning-of-line)
@@ -611,6 +615,8 @@ address string extracted."
 	(progn
 	  (cmail-fixcp)
 	  (cmail-send-digest user))
+	(with-temp-buffer
+	  (let ((*cmail-mail-buffer (buffer-name)))
       (cmail-fixcp)
       (cmail-get-folder)
       (cmail-n-page page)
@@ -623,16 +629,21 @@ address string extracted."
       (let ((cmail-mime-automatic-preview cmail-mime-forwarding))
 	(save-excursion
 	  (cmail-show-contents page cmail-mime-forwarding)))
-      (cmail-mail user subject)
+      (cmail-mail user subject 'forward)
       (set-buffer "*mail*")
-      (make-local-variable 'truncate-partial-width-windows)
-      (setq truncate-partial-width-windows nil)
       (save-excursion
-	(goto-char (point-min))
-	(search-forward "\n\n" nil 1)
+	(when (or (re-search-forward mail-header-separator nil t)
+		  (when (save-excursion
+			  (beginning-of-line 0)
+			  (looking-at mail-header-separator))
+		    (end-of-line 0)
+		    t))
+	  (while (and (search-forward "\n\n" nil 1)
+		      (invisible-p (point)))
+	    (next-visible-point (point))))
 	(if (and cmail-use-mime cmail-mime-forwarding)
 	    (cmail-mime-insert-forward-message)
-	  (cmail-insert-forward-message))))))
+	  (cmail-insert-forward-message))))))))
 
 (defun cmail-alt-forward (arg)
   "$B%+%l%s%H5-;v$r(Bcmail-mime-forwarding$B$N;XDj$G$O$J$$J}K!$G%U%)%o!<%I$9$k(B."
@@ -945,8 +956,6 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
     (cmail-mail "" (cmail-format-resource 'resend-bounced-mail-2
 					  cmail-current-folder page))
     (set-buffer "*mail*")
-    (make-local-variable 'truncate-partial-width-windows)
-    (setq truncate-partial-width-windows nil)
     (erase-buffer)
     (cmail-insert-buffer-substring b start lim)
     (cmail-ignore-headers)
@@ -966,7 +975,9 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
 $B%5%^%j%P%C%U%!$K%+!<%=%k$rLa$9(B."
   (interactive "bKill buffer: ")
   (kill-buffer buf)
-  (cmail-go-summary))
+  (let ((sbuf (get-buffer *cmail-summary-buffer)))
+    (if (and sbuf (buffer-live-p sbuf))
+	(cmail-go-summary))))
 
 ;;;
 ;;;
@@ -1060,10 +1071,10 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
 	(save-excursion
 	(set-buffer cur)
 	(or (and cmail-original-message-end
-			 (setq bottom
-				   (and (re-search-forward
-						 cmail-original-message-end nil (point-max))
-						(match-beginning 0))))
+		 (setq bottom
+		       (and (re-search-forward
+			     cmail-original-message-end nil (point-max))
+			    (match-beginning 0))))
 		(setq bottom (point-max)))
 	(save-restriction
 	  (narrow-to-region top bottom)
@@ -1092,25 +1103,27 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
 ;; -->
 ;; should be added before cmail-mime add its hook functions.
 (defvar cmail-unify-multi-re-in-subject nil)
-(add-hook 'cmail-mail-hook
-	  '(lambda ()
-	     (when cmail-unify-multi-re-in-subject
-	       (save-excursion
-		 (setq cmail-unify-multi-re-in-subject nil)
-		 (goto-char (point-min))
-		 (when (re-search-forward "^Subject: \\(.+\\)$" nil t)
-		   (let ((sbj (buffer-substring (match-beginning 1) (match-end 1))))
-		     (delete-region (match-beginning 1) (match-end 1))
-		     (if (not (or cmail-remove-re
-				  cmail-unify-multi-re ; unify $B$7$J$$$H$-$d(B Re: $B$,$J$$>l9g$O(B
-				  (string-match cmail-re-regexp sbj)))
-			 (setq sbj (concat "Re: " sbj)) ; $BC1=c$K(B Re: $B$rIU$1$k(B
-		       (if (string-match cmail-re-regexp sbj) ; Re: $B$,$9$G$K$"$k>l9g$O(B
-			   (setq sbj (substring sbj (match-end 0)))) ; $B$=$l$r:o$C$F$+$i(B
-		       (or cmail-remove-re
-			   (setq sbj (concat "Re: " sbj)))) ; Re: $B$rIU$1$k(B
-		     (insert sbj)
-		     ))))))
+(add-hook
+ 'cmail-mail-hook
+ '(lambda ()
+    (when cmail-unify-multi-re-in-subject
+      (save-excursion
+	(setq cmail-unify-multi-re-in-subject nil)
+	(goto-char (point-min))
+	(when (re-search-forward "^Subject:\\s *\\(.+\\)$" nil t)
+	  (let ((sbj (buffer-substring (match-beginning 1) (match-end 1))))
+	    ;; $B%f!<%6$,L@<(E*$KJQ99$7$?2DG=@-$,$"$l$P(B Re $B$rIU$1$J$$!#(B
+	    (unless noerase
+	      (delete-region (match-beginning 1) (match-end 1))
+	      (if (not (or cmail-remove-re
+			   cmail-unify-multi-re ; unify $B$7$J$$$H$-$d(B Re: $B$,$J$$>l9g$O(B
+			   (string-match cmail-re-regexp sbj)))
+		  (setq sbj (concat "Re: " sbj)) ; $BC1=c$K(B Re: $B$rIU$1$k(B
+		(if (string-match cmail-re-regexp sbj) ; Re: $B$,$9$G$K$"$k>l9g$O(B
+		    (setq sbj (substring sbj (match-end 0)))) ; $B$=$l$r:o$C$F$+$i(B
+		(or cmail-remove-re
+		    (setq sbj (concat "Re: " sbj)))) ; Re: $B$rIU$1$k(B
+	      (insert sbj))))))))
 ;; <--
 
 ;;;
@@ -1125,6 +1138,9 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
   "Hook for cmail draft list mode")
 (defvar cmail-draft-list-mode-map nil
   "Mode map for cmail draft list mode")
+(defvar cmail-get-draft-message-listing-func
+  '(cmail-draft-list-buffers))
+(defvar cmail-draft-message-list nil)
 
 (cond
  ((null cmail-draft-list-mode-map)
@@ -1152,115 +1168,157 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
   (cmail-draft-list-mode))
 
 (defun cmail-draft-list-mode ()
-  "
-cmail $B%a%C%;!<%8:n@.%P%C%U%!$N%j%9%H$rI=<($7$FA*Br$9$k$3$H$,$G$-$^$9!#(B
+  "cmail $B%a%C%;!<%8:n@.%P%C%U%!$N%j%9%H$rI=<($7$FA*Br$9$k$3$H$,$G$-$^$9!#(B
 
-\\{cmail-draft-list-mode-map}
-"
+\\{cmail-draft-list-mode-map}"
   (interactive)
   (kill-all-local-variables)
   (setq major-mode 'cmail-draft-list-mode)
   (setq mode-name "cmail draft list")
   (use-local-map cmail-draft-list-mode-map)
   (setq buffer-read-only nil)
-  (cmail-draft-format-buffer (current-buffer)
-			     (cmail-get-draft-buffer-list))
+  (make-local-variable 'cmail-draft-message-list)
+  (setq cmail-draft-message-list (cmail-get-draft-message-list))
+  (cmail-draft-format-buffer (current-buffer) cmail-draft-message-list)
   (setq truncate-lines t)
   (setq buffer-read-only t)
   (goto-char (point-min))
   (forward-line 2)
   (run-hooks 'cmail-draft-list-mode-hook))
 
+(defun cmail-get-draft-message-list ()
+  (let ((list-funcs cmail-get-draft-message-listing-func)
+	(msgnum 1) res1 res)
+    (while list-funcs
+      (setq res1 (apply (car list-funcs) (list msgnum)))
+      (setq list-funcs (cdr list-funcs))
+      (if res1
+	  (progn
+	    (setq msgnum (car res1))
+	    (setq res (append res (list (cdr res1)))))))
+    res))
+
 (defun cmail-draft-format-buffer (buffer draft-list)
-  "
-Format cmail draft list buffer including header."
-  (let (buf-list buf)
+  "Format cmail draft list buffer including header."
+  (let (msg-list)
     (save-excursion
       (set-buffer buffer)
       (delete-region (point-min) (point-max))
-      (make-local-variable 'cmail-draft-buffer-list)
-      (setq cmail-draft-buffer-list draft-list)
-      (cmail-draft-insert-line "Buffer" "To" "Subject")
-      (cmail-draft-insert-line "------" "--" "-------")
+      (cmail-draft-insert-line "MsgID" "To" "Subject")
+      (cmail-draft-insert-line "-----" "--" "-------")
       (cmail-draft-insert-line "[new]" "" "")
       (while draft-list
-	(setq buf (car draft-list))
-	(apply 'cmail-draft-insert-line
-	       (buffer-name (car buf))
-	       (cdr (cdr buf)))
+	(setq msg-list (cdr (car draft-list)))
+	(if msg-list
+	    (progn
+	      (insert (concat ": ===== " (car (car draft-list)) " =====\n"))
+	      (while msg-list
+		(apply 'cmail-draft-insert-line
+		       (list (nth 0 (car msg-list))
+			     (nth 1 (car msg-list))
+			     (nth 2 (car msg-list))))
+		(setq msg-list (cdr msg-list)))))
 	(setq draft-list (cdr draft-list))))))
 
-(defun cmail-get-draft-buffer-list ()
-  "
-Create a cmail draft buffers list by going through the list of
-buffers. It looks for buffers in cmail-mail-mode or mail-mode. The
-cmail draft buffers list returned from this function is a list of
-mail mode buffer information list, which includes a buffer object,
-buffer filename, 'to' field content, and 'subject' field content."
-  (save-excursion
-    (let (res (bl (buffer-list)))
-      (while bl
-	(set-buffer (car bl))
-	(goto-char (point-min))
-	(if (or (eq major-mode 'mail-mode)
-		(eq major-mode 'cmail-mail-mode))
-	    (setq res
-		  (append
-		   (list (list
-			  (current-buffer)
-			  (or (buffer-file-name) "")
-			  (or (cmail-get-field-values "To" ", ") "")
-			  (or (cmail-get-field-values "Subject") "")))
-		   res)))
-	(setq bl (cdr bl)))
-      res)))
-
-(defun cmail-draft-insert-line (bufname to subject)
-  "
-Insert a formated entry of a line in cmail draft list mode buffer."
+(defun cmail-draft-insert-line (index to subject)
+  "Insert a formated entry of a line in cmail draft list mode buffer."
   (insert (format "  %s %s %s\n"
-		  (cmail-fill -20 bufname)
+		  (cmail-fill 5 (if (stringp index)
+				    index
+				  (int-to-string index)))
 		  (cmail-fill -24 to)
 		  subject)))
 
 ;;; draft mode commands
-(defun cmail-draft-list-open ()
-  "$B8=:_$N%+!<%=%k$N$"$k9T$KBP1~$9$k%a!<%k:n@.%P%C%U%!$r8F$S=P$7$^$9!#(B"
-  (interactive)
-  (let ((buffer (cmail-draft-list-get-buffer)))
-    (cond ((bufferp buffer)
-	   (switch-to-buffer buffer))
-	  ((eq buffer 'new)
-	   (cmail-mail "" "")))))
-
 (defun cmail-draft-list-quit ()
   "$B%a!<%k:n@.%P%C%U%!%j%9%H%b!<%I$r=*N;$7$^$9!#(B"
   (interactive)
-  (kill-buffer (current-buffer)))
+  (kill-buffer (current-buffer))
+  (cmail-go-summary))
+
+(defun cmail-draft-list-open ()
+  "$B8=:_$N%+!<%=%k$N$"$k9T$KBP1~$9$k%a!<%k:n@.%P%C%U%!$r8F$S=P$7$^$9!#(B"
+  (interactive)
+  (let ((message (cmail-draft-list-get-draft-message)))
+    (cond ((and message (listp message))
+	   (apply (nth 3 message) (list 'open message)))
+	  ((eq message 'new)
+	   (cmail-mail "" ""))
+	  (t (message "No valid draft message.")))))
 
 (defun cmail-draft-list-delete ()
   "$B8=:_$N%+!<%=%k$N$"$k9T$KBP1~$9$k%a!<%k:n@.%P%C%U%!$r:o=|$7$^$9!#(B"
   (interactive)
-  (let ((buffer (cmail-draft-list-get-buffer)))
-    (if (bufferp buffer)
-	(progn
-	  (kill-buffer buffer)
-	  (let ((point (point)))
-	    (cmail-draft-list-mode)
-	    (goto-char point)))
-      (message "No valid buffer to delete."))))
+  (let ((message (cmail-draft-list-get-draft-message)))
+    (cond ((and message (listp message))
+	   (apply (nth 3 message) (list 'delete message)))
+	  (t (message "No valid draft message.")))))
 
-(defun cmail-draft-list-get-buffer ()
-  "
-Return buffer name corresponding to the current cursor line. This
+(defun cmail-draft-list-get-draft-message ()
+  "Return buffer name corresponding to the current cursor line. This
 function should be used in a cmail draft list mode buffer. If the line
 does not contain a valid buffer name at the proper position in the
 line, then checks if a special contorl line. ex. if it finds [new],
 return a symbol ' new. Otherwise, it returns nil."
   (save-excursion
     (beginning-of-line)
-    (if (re-search-forward "^  \\([^ ][^ ]*\\)" nil t)
-	(let ((buffer-name (buffer-substring (match-beginning 1)
-					     (match-end 1))))
-	  (or (get-buffer buffer-name)
-	      (and (string-match "\[new\]" buffer-name) 'new))))))
+    (if (looking-at " *\\([0-9]+\\)")
+	(let ((id (string-to-int (buffer-substring (match-beginning 1)
+						   (match-end 1)))))
+	  (if (> id 0)
+	      (assoc id (apply 'append
+			       (mapcar 'cdr cmail-draft-message-list)))))
+      (if (looking-at "  \\[new\\]") 'new))))
+
+;;;
+;;; draft list for mail mode buffers
+;;;
+(defun cmail-draft-list-buffers (msgnum)
+  "
+Create a cmail draft message list by going through the list of
+buffers. It looks for buffers in cmail-mail-mode or mail-mode. The
+cmail draft buffers list returned from this function is a list of
+mail mode buffer information list, which includes a buffer object,
+buffer filename, 'to' field content, and 'subject' field content."
+  (save-excursion
+    (let ((bl (buffer-list)) res1 res)
+      (while bl
+	(set-buffer (car bl))
+	(if (or (eq major-mode 'mail-mode)
+		(eq major-mode 'cmail-mail-mode))
+	    (progn
+	      (goto-char (point-min))
+	      (setq res1
+		    (append
+		     res1
+		     (list (list
+			    msgnum
+			    (or (cmail-get-field-values-in-mail-buffer "To" ", ") "")
+			    (or (cmail-get-field-values-in-mail-buffer "Subject") "")
+			    'cmail-draft-list-buffer-commands
+			    (current-buffer)
+			    (or (buffer-file-name) "")))))
+	      (setq msgnum (1+ msgnum))))
+	(setq bl (cdr bl)))
+      (setq res (cons "Mail Mode Buffers" res1))
+      (cons msgnum res))))
+
+(defconst cmail-draft-list-buffer-commands
+  '((open . cmail-draft-list-buffer-open)
+    (delete . cmail-draft-list-buffer-delete) ))
+    
+(defun cmail-draft-list-buffer-commands (operation message)
+  (let ((func (cdr (assoc operation cmail-draft-list-buffer-commands))))
+    (if func (apply func (list message)))))
+
+(defun cmail-draft-list-buffer-open (message)
+  (bury-buffer)
+  (switch-to-buffer (nth 4 message)))
+
+(defun cmail-draft-list-buffer-delete (message)
+  (kill-buffer (nth 4 message))
+  (cmail-draft-list-mode)
+  (beginning-of-line))
+
+;;;
+;;;
