@@ -509,7 +509,7 @@
 (defun cmail-sort-by-date (&optional quiet)
   "$BF|IU=g$K%5%^%j$r%=!<%H$9$k(B."
   (interactive)
-  (cmail-sort-internal "\\3" quiet))
+  (cmail-sort-by-time quiet))
 
 (defun cmail-sort-by-author (&optional quiet)
   "$BAw$j<g$NL>A0=g$K%5%^%j$r%=!<%H$9$k(B."
