@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-vars.el - cmail variables initialization
 ;;;
-;;;  $Author: iwa $
+;;;  $Author: MaNakagawa $
 ;;;  created at: Mon Jul 12 10:30:57 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto.
@@ -24,8 +24,9 @@
 ;; and this notice must be preserved on all copies.
 
 (provide 'cmail-vars)
-(require 'rfc822)
-(require 'pcustom)
+;(require 'rfc822)
+(eval-when-compile (require 'pcustom)
+		   (require 'static))
 
 ;;
 ;; ======= customize $B%7%g!<%H%+%C%H%3%^%s%I(B =============================
@@ -37,9 +38,67 @@
 	(delete-other-windows))
     (error "Sorry, but the custom feature is not available on this system.")))
 
+;;
+;; ======= $B%3%s%Q%A%S%j%F%#$N3NJ](B =======================================
+
+;; In case when a custom version that does not support :set feature is
+;; loaded, to avoid a problem with compiling/running cmail, here goes
+;; some workaround definitions. First, for those Emacs that do not
+;; support keywords (symbols that begin with a colon), define them for
+;; custom use.
+(static-if (null (boundp ':cmail-custom-test-symbol))
+    (mapcar
+     '(lambda (keyword) (or (boundp keyword) (set-default keyword keyword)))
+     '(:initialize :set :get :require :prefix :tag
+		   :load :link :options :type :group)))
+
+;; Now, test defcustom if it has the :set feature, and if it does not
+;; behave as expected, override defcustom with defvar like tinycustom
+;; does. Note that all of these should happen at compile time to limit
+;; the scope in order to avoid changing run-time behavior of other
+;; packages which assume actual custom (such as Gnus). If this module
+;; is loaded or evaluated as a lisp source, then the change will be in
+;; effect in a global manner, which may cause some trouble to those
+;; packages. Please avoid loading or evaluating the source version of
+;; this file unless necessary.
+(static-if (let (test-sym)
+	     (condition-case nil
+		 (defcustom *cmail-custom-test-symbol t ""
+		   :set '(lambda (sym val) (setq test-sym val)))
+	       (error nil))
+	     test-sym)
+    (defmacro cmail-custom-add-init (init-function sym) nil)
+  (eval-when-compile
+    (defmacro defcustom (sym val doc &rest rest)
+      "An emulated defcustom in cmail-vars.el, when byte-compile only."
+      (` (defvar (, sym) (, val) (,doc)))))
+  
+  ;; For those customizable variables that supports the defcustom
+  ;; :set feature (except for cmail-i18n-doc-language), when
+  ;; 'tiny' version of defcustom is used, initialization must be
+  ;; explicitly executed.  The following macro can be used to set
+  ;; up the initializer for such variables, and cmail-custom-init,
+  ;; which is called from the function cmail, will initialize the
+  ;; instrumented variables.
+  (defvar *cmail-custom-initializer nil)
+  (defmacro cmail-custom-add-init (init-function sym)
+    (` (progn
+	 (setq *cmail-custom-initializer
+	       (append *cmail-custom-initializer
+		       (list (list (, init-function) (, sym))))))))
+  
+  (defun cmail-custom-init ()
+    (mapcar
+     (function
+      (lambda (init)
+	(if (boundp (elt init 1))
+	    (funcall (elt init 0) (elt init 1) (eval (elt init 1))))))
+     *cmail-custom-initializer)))
+
 ;;
 ;; ======= $B%I%-%e%a%s%H$NJ#?t8@8l$N%5%]!<%H(B =============================
 (defvar cmail-doc-custgroup nil)
+(put 'cmail-i18n-defgroup 'lisp-indent-function 2)
 (defmacro cmail-i18n-defgroup (group something doc &rest rest)
   (` (prog1
 	 (, (append (` (defgroup (, group) (, something)
@@ -50,6 +109,7 @@
 			    cmail-doc-custgroup))))))))
 
 (defvar cmail-doc-custvars nil)
+(put 'cmail-i18n-defcustom 'lisp-indent-function 2)
 (defmacro cmail-i18n-defcustom (name value doc &rest rest)
   (` (prog1
 	 (, (append (` (defcustom (, name) (, value)
@@ -60,7 +120,7 @@
 			    cmail-doc-custvars))))))))
 
 (defun cmail-i18n-set-doc-language (symbol language)
-  (setq cmail-i18n-doc-language language)
+  (set symbol language)
   (cmail-i18n-swap-doc language))
 
 (defun cmail-i18n-swap-doc (language)
@@ -1081,7 +1141,7 @@ extra information string in the address fields."))
 ;; --- Copied from rmail.el ---
 ;; Some mailers use "Re(2):" or "Re^2:" or "Re: Re:" or "Re[2]:".
 ;; This pattern should catch all the common variants.
-(cmail-i18n-defcustom cmail-re-regexp "\\`\\(Re\\(([0-9]+)\\|\\[[0-9]+\\]\\|\\^[0-9]+\\)?: *\\)*"
+(cmail-i18n-defcustom cmail-re-regexp "\\`\\(Re\\(([0-9]+)\\|\\[[0-9]+\\]\\|\\^[0-9]+\\)?: *\\)+"
   ((ja_JP . "\
 *Re: $B$r$D$1$kA0$K:o=|$9$kJ8;zNs$K%^%C%A$9$k@55,I=8=(B.")
    (en_US . "\
@@ -1640,14 +1700,23 @@ Menu title will show the current message."))
   :group 'cmail-menu-style)
 
 (cmail-i18n-defcustom cmail-sum-pop-up-menu-1
-  '(("Get New Message" . cmail-summary-get-newmail)
-    ("Folders List" . cmail-folders)
-    ("Expand Window" . cmail-sum-mouse-expand)
-    ("Archive" . cmail-auto-archive)
-    ("Expunge" . cmail-execute-index)
-    ("Save All" . cmail-save-all-folders)
-    ""
-    ("Quit" . cmail-summary-quit))
+  `(,@(if running-xemacs
+	  '(["Get New Message" cmail-summary-get-newmail t]
+	    ["Folders List" cmail-folders t]
+	    ["Expand Window" cmail-sum-mouse-expand t]
+	    ["Archive" cmail-auto-archive t]
+	    ["Expunge" cmail-execute-index t]
+	    ["Save All" cmail-save-all-folders t]
+	    "---"
+	    ["Quit" cmail-summary-quit t])
+	'(("Get New Message" . cmail-summary-get-newmail)
+	  ("Folders List" . cmail-folders)
+	  ("Expand Window" . cmail-sum-mouse-expand)
+	  ("Archive" . cmail-auto-archive)
+	  ("Expunge" . cmail-execute-index)
+	  ("Save All" . cmail-save-all-folders)
+	  ""
+	  ("Quit" . cmail-summary-quit))))
   ((ja_JP . "\
 *$B%5%^%j$GDL>o(B3$BHVL\$N%^%&%9%\%?%s$N%]%C%W%"%C%W%a%K%e!<(B.
 
@@ -1660,6 +1729,45 @@ Menu title will show the current folder name."))
   :group 'cmail-all-variables
   :group 'cmail-menu-style)
 
+(cmail-i18n-defcustom cmail-option-path-list
+		      (list (concat
+			     (file-name-directory (locate-library "cmail"))
+			     "options/"))
+  ((ja_JP . "\
+*$B%*%W%7%g%s5!G=MQ$N%U%!%$%k$rCV$/%G%#%/%H%j!#(B")
+   (en_US . "\
+*A direcoty for files providing optional features."))
+  :type '(repeat directory)
+  :group 'cmail-all-variables
+  :group 'cmail-directories)
+
+(cmail-i18n-defcustom cmail-auth-file "~/.cmail-auth"
+  ((ja_JP . "\
+*POP$B$J$I$N%Q%9%o!<%I$rJ]B8$9$k%U%!%$%kL>!#(B
+
+cmail-use-auth-file $B$,(B non nil $B$N;~$KM-8z$K$J$k!#(B")
+   (en_US . "\
+*Name of file to store password for mail retrieval protocol such as POP.
+
+Used when cmail-use-auth-file is non nil."))
+  :type 'directory
+  :group 'cmail-all-variables
+  :group 'cmail-directories)
+
+(cmail-i18n-defcustom cmail-use-auth-file nil
+  ((ja_JP . "\
+*Non nil $B$N;~%Q%9%o!<%I$r(B cmail-auth-file $B$G;XDj$N%U%!%$%k$KJ]B8$9$k!#(B
+
+$B%Y!<%9%7%9%F%`$,%U%!%$%k$N5v2D%b!<%I$r%5%]!<%H$7$F$$$J$$$H%U%!%$%k$O(B
+$B:n@.$5$l$^$;$s!#(B")
+   (en_US . "\
+*Non nil means store passwords in a file specified by cmail-auth-file.
+
+The system must support file permission mode to create the auth file."))
+  :type 'boolean
+  :group 'cmail-all-variables
+  :group 'cmail-directories)
+
 
 ;; ==== $B:n6HMQJQ?t(B($B5Z$SDj?t(B) ====
 
@@ -1767,6 +1875,6 @@ Menu title will show the current folder name."))
 
 ;;
 ;; ======= $B:G8e$K%I%-%e%a%s%H$rF~$l49$((B =================================
-(cmail-i18n-swap-doc cmail-i18n-doc-language)
-
+;(cmail-i18n-swap-doc cmail-i18n-doc-language)
+;-> cmail.el $B$N:G8e$X(B
 ;; ======================================================================
