@@ -1,9 +1,9 @@
 ;;;
 ;;;  cmail.el - a mail utility for GNU Emacs
 ;;;
-;;;  $Author: iwa $
-;;;  $Revision: 1.6 $
-;;;  $Date: 1999/10/29 04:41:41 $
+;;;  $Author: MaNakagawa $
+;;;  $Revision: 1.15 $
+;;;  $Date: 2000/10/03 15:46:23 $
 ;;;  created at: Mon Jul 12 10:09:15 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1998 Yukihiro Matsumoto
@@ -43,15 +43,9 @@
   cmail-version)
 
 (provide 'cmail)
-(require 'cl)
 
+(require 'cl)
 (require 'emu)
-(or (functionp 'decode-coding-region)
-    (defun decode-coding-region (start end coding-system)
-      "Decode the text between START and END which is encoded in CODING-SYSTEM.
-\[EMACS 20 emulating function]"
-      (code-convert-region start end coding-system *internal*))
-    )
 
 (require 'cmail-vars)
 (require 'cmail-modes)
@@ -100,6 +94,7 @@
 prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
 $B;XDj$5$l$?%U%)%k%@$r3+$/(B."
   (interactive "P")
+  (if (functionp 'cmail-custom-init) (cmail-custom-init))
   (or cmail-use-full-window
       (setq *cmail-window-conf (current-window-configuration)))
   (cmail-start confirm)
@@ -188,130 +183,25 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
 
 (defun cmail-get-spooled-mail ()
   "$B%9%W!<%k$+$i%a%$%k$r<h$C$F$/$k(B."
-  (let (popmail
-	(inhibit-quit t)
-	(mails 0)
-	err spool
-	pop-host			; popmail $B$,(B t $B$N$H$-$OI,$:(B pop-host
-					; $B$,(B non-nil $B$K$J$k$h$&$K$9$k(B
-
-					; $BJ#?t(BPOP$B%5!<%P@_Dj$N$H$-$K4D6-JQ?t$,(B
-					; $B=q$-49$($i$l$F$7$^$&$N$OB>$N%=%U%H$K(B
-					; $B1F6A$,$"$k$+$b$7$l$J$$$N$G!"J]B8(B
-	(default-mailhost (getenv "MAILHOST"))
-	(default-movemail (getenv "MOVEMAIL"))
-	(spools cmail-mail-spools))
+  (let ((inhibit-quit t)
+	(account-list (cmail-normalize-account-info))
+	(mails 0))
     (cmail-open-folder cmail-inbox-folder)
     (if (cmail-check-crash-box)
 	(progn
 	  (cmail-message-resource 'get-spooled-mail-1)
 	  (setq mails (cmail-gobble-crash-box))
 	  (cmail-message-resource 'get-spooled-mail-2)))
+    (while account-list
+      ;; fetch message with specified protocol with
+      ;; corresponding protocol handler.
+      (cmail-fetch-messages *cmail-crash-box (car account-list))
 
-					; cmail-mail-spools $B$,Dj5A$5$l$F$$$J$$(B
-					; $B>l9g!"B>$N>pJs$r$b$H$KF1$87A<0$N%j%9(B
-					; $B%H$r:n@.$7$F(B spools $B$KF~$l$k(B
-    (if (null spools)
-	(let (mf)
-	  (setq mf (getenv "MAIL"))
-	  (if (null mf)
-	      (progn
-		(setq mf cmail-spool-directory)
-		(or (string-match "^po:" mf)
-		    (setq mf (concat mf (user-login-name))))))
-	  (setq spools (list mf)))
-      (if (stringp spools)
-	  (setq spools (list spools))))
-
-    (while spools			; spools $B$N$=$l$>$l$NMWAG$K4X$7$F%k!<%W(B
-      (setq spool (car spools))
-      (cond
-       ((listp spool)			; ("po:foo" "host" "option" "passwd")
-	(setq popmail t)		; $B$N7A<0$N>l9g(B
-	(setq pop-host (cadr spool))
-	(setenv "MOVEMAIL" (caddr spool))
-	(setq *cmail-pop-password (cadddr spool))
-	(setq spool (car spool))
-	(setenv "MAILHOST" pop-host))
-
-       ((string-match "^po:" spool)	; "po:user" $B$N7A<0$N>l9g(B (POP)
-	(setq popmail t)
-	(setq pop-host default-mailhost) ; (getenv "MAILHOST") $B$G$O%@%a(B
-	(setenv "MAILHOST" default-mailhost)
-	(setenv "MOVEMAIL" default-movemail)
-	(and (or (featurep 'cmail-e19) (featurep 'cmail-20))
-	     (string-match "@" spool)
-	     (setq pop-host (substring spool (1+ (match-beginning 0)))))
-	(if (null pop-host)
-	    (cmail-error-resource 'get-spooled-mail-12)))
-
-       (t				; "$B%9%W!<%k%U%!%$%kL>(B" $B$N>l9g(B
-	(setq popmail nil)
-	(setq spool (expand-file-name spool))))
-
-					; POP $B$G!"%Q%9%o!<%I$,(B nil $B$N>l9g$O(B
-					; $B%f!<%6$KJ9$/(B
-      (if (and popmail (null *cmail-pop-password)) ; modified by himi to get pop password
-	  (setq *cmail-pop-password
-		(let (hmesg pass)
-		  (setq hmesg (cmail-format-resource1 'get-spooled-mail-4 pop-host))
-		  (require 'comint)
-		  (setq pass (comint-read-noecho hmesg t))
-		  (message "")
-		  pass)))
-
-      (if (or popmail
-	      (and (file-exists-p spool)
-		   (not (equal 0 (nth 7 (file-attributes spool))))
-		   (file-readable-p spool)))
-	  (let ((errors nil))
-	    (if popmail
-		(cmail-message-resource1 'get-spooled-mail-5 pop-host)
-	      (cmail-message-resource1 'get-spooled-mail-7 spool))
-	    (unwind-protect
-		(save-excursion
-		  (setq errors (generate-new-buffer " *cmail loss*"))
-		  (buffer-flush-undo errors)
-		  (if (and popmail *cmail-pop-password)
-		      (call-process *cmail-movemail-command nil errors nil
-				    spool (expand-file-name *cmail-crash-box) 
-				    *cmail-pop-password)
-		    (call-process *cmail-movemail-command nil errors nil
-				  spool (expand-file-name *cmail-crash-box)))
-		  (if (not (buffer-modified-p errors))
-		      nil		; No output => movemail won
-		    (setq err t)
-		    (setq *cmail-pop-password nil)
-		    (set-buffer errors)
-		    (subst-char-in-region (point-min) (point-max)
-					  ?\n ?\  )
-		    (goto-char (point-max))
-		    (skip-chars-backward " \t")
-		    (delete-region (point) (point-max))
-		    (goto-char (point-min))
-		    (if (looking-at "movemail: ")
-			(delete-region (point-min) (match-end 0)))
-		    (message "%s" (concat "movemail: "
-					  (buffer-substring (point-min)
-							    (point-max))))))
-	      (if errors (kill-buffer errors)))
-	    (if (cmail-check-crash-box)
-		(setq mails (cmail-gobble-crash-box)))
-	    (if err
-		(setq *cmail-pop-password nil)
-	      (if popmail
-		  (cmail-message-resource1 'get-spooled-mail-6 pop-host)
-		(cmail-message-resource1 'get-spooled-mail-8 spool)))))
-      (if (listp (car spools))
-	  (let ((c (car spools)))
-	    (setcar (cdddr c) *cmail-pop-password)
-	    (setq *cmail-pop-password nil)))
-      (setq spools (cdr spools)))
-
-    (setenv "MAILHOST" default-mailhost)
-    (setenv "MOVEMAIL" default-movemail)
-    (or (> mails 0)
-        err
+      ;; convert format and add it to cmail inbox
+      (if (cmail-check-crash-box)
+	  (setq mails (+ mails (cmail-gobble-crash-box))))
+      (setq account-list (cdr account-list)))
+    (if (= mails 0)
 	(cmail-message-resource 'get-spooled-mail-9))
     (> mails 0)))
 
@@ -403,6 +293,330 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
       (kill-buffer *cmail-arrived-mail-buffer)
       cnt)))
 
+;; ---- $B%W%m%H%3%k<h9~$_4X?t$N%F!<%V%kA`:n4X?t(B -------------------------
+(defconst *cmail-fetch-default-handlers
+  '((local-file . cmail-movemail-spool)
+    (pop3 . cmail-movemail-pop3)))
+
+(defvar *cmail-fetch-handlers
+  (mapcar (function (lambda (element) (cons (car element) (cdr element))))
+	  *cmail-fetch-default-handlers))
+
+(defun cmail-fetch-messages (crash-box account-spec)
+  (let ((handler (cdr (assoc (car account-spec) *cmail-fetch-handlers))))
+    (if handler
+	(apply handler (list crash-box account-spec)))))
+
+(defun cmail-add-fetch-handler (proto-label handler-function)
+  (let ((rec (assoc proto-label *cmail-fetch-handlers)))
+    (if rec
+	(setcdr rec handler-function))))
+
+(defun cmail-del-fetch-handler (proto-label handler-function)
+  (let ((rec (assoc proto-label *cmail-fetch-handlers))
+	(def-rec (assoc proto-label *cmail-fetch-default-handlers)))
+    (if (and rec (eq (cdr rec) handler-function) def-rec)
+	(setcdr rec (cdr def-rec))
+      (cdr rec))))
+
+;; ---- $BI8=`%a!<%k$N<h9~$_%O%s%I%i(B (movemail) ----------------------------
+(defun cmail-movemail-spool (crash-box proto-spec)
+  "$BI8=`%a!<%k$N<h9~$_%O%s%I%i(B ($B%m!<%+%k%9%W!<%k%U%!%$%kMQ(B)
+$B30It%W%m%;%9(B movemail $B$rMxMQ$9$k!#(B
+
+PROTO-SPEC $B$O(B (local-file \"filename\") $B$N%j%9%H!#(B"
+  (let ((spool-file (elt proto-spec 1)))
+    (cmail-message-resource1 'get-spooled-mail-7 spool-file)
+    (unwind-protect
+      (save-excursion
+	  (setq errors (generate-new-buffer " *cmail loss*"))
+	  (buffer-flush-undo errors)
+	  (call-process *cmail-movemail-command nil errors nil
+			spool-file (expand-file-name *cmail-crash-box))
+	  (if (not (buffer-modified-p errors))
+	      ;; No output => movemail won
+	      (cmail-message-resource1 'get-spooled-mail-8 spool-file)
+	    (set-buffer errors)
+	    (subst-char-in-region (point-min) (point-max) ?\n ?\  )
+	    (goto-char (point-max))
+	    (skip-chars-backward " \t")
+	    (delete-region (point) (point-max))
+	    (goto-char (point-min))
+	    (if (looking-at "movemail: ")
+		(delete-region (point-min) (match-end 0)))
+	    (message "%s" (concat "movemail: "
+				  (buffer-substring (point-min)
+						    (point-max))))
+	    (sit-for 1)))
+      (if errors (kill-buffer errors)))))
+
+(defun cmail-movemail-pop3 (crash-box proto-spec)
+  "$BI8=`%a!<%k$N<h9~$_%O%s%I%i(B (POP3 $BMQ(B)
+$B30It%W%m%;%9(B movemail $B$rMxMQ$9$k!#(B
+PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
+$B$N%j%9%H!#(B"
+  (let ((server (elt proto-spec 1))
+	(logname (elt proto-spec 2))
+	(password (elt proto-spec 3))
+	(options (elt proto-spec 4))
+	(default-mailhost (getenv "MAILHOST"))
+	(default-movemail (getenv "MOVEMAIL")))
+    (if (null password)
+	(setq password (cmail-lookup-password 'pop3 server logname)))
+    (if (or (null password) (string= password ""))
+	(progn
+	  (setq password (cmail-prompt-for-password server logname))
+	  (cmail-save-password 'pop3 server logname password)))
+    (if (and (stringp password) (not (string= password "")))
+	(progn
+	  (setenv "MAILHOST" server)
+	  (setenv "MOVEMAIL" options)
+	  (cmail-message-resource1 'get-spooled-mail-5 server)
+	  (unwind-protect
+	      (save-excursion
+		(setq errors (generate-new-buffer " *cmail loss*"))
+		(buffer-flush-undo errors)
+		(call-process *cmail-movemail-command nil errors nil
+			      (concat "po:" logname)
+			      (expand-file-name *cmail-crash-box)
+			      password)
+		(if (not (buffer-modified-p errors))
+		    ;; No output => movemail won
+		    (cmail-message-resource1 'get-spooled-mail-6 server)
+		  (cmail-save-password 'pop3 server logname "")
+		  (set-buffer errors)
+		  (subst-char-in-region (point-min) (point-max) ?\n ?\  )
+		  (goto-char (point-max))
+		  (skip-chars-backward " \t")
+		  (delete-region (point) (point-max))
+		  (goto-char (point-min))
+		  (if (looking-at "movemail: ")
+		      (delete-region (point-min) (match-end 0)))
+		  (message "%s" (concat "movemail: "
+					(buffer-substring (point-min)
+							  (point-max))))
+		  (sit-for 1)))
+	    (if errors (kill-buffer errors)))
+	  (setenv "MAILHOST" default-mailhost)
+	  (setenv "MOVEMAIL" default-movemail)))))
+
+
+;; ---- $B%"%+%&%s%H>pJsA`:n4X?t(B --------------------------------------------
+(defun cmail-normalize-account-info ()
+  "$B%a!<%k$N<h$j9~$_@h(B($B%"%+%&%s%H(B)$BKh$K%W%m%H%3%k$r(B cmail-mail-spools $B$*(B
+$B$h$S(B cmail-spool-directory $B$r$b$H$K?dB,$7$=$l$>$l$N<1JL$5$l$?%W%m%H%3(B
+$B%kKh$K7h$a$i$l$?>pJs$r%j%9%H$7$FJV$9!#0z?t$K%W%m%H%3%k$r;XDj$9$k$H$=$N(B
+$B%W%m%H%3%k$N$_$N>pJs$rJV$9!#0z?t$G$N%W%m%H%3%k$OJ#?t;XDj$G$-$k!#(B"
+  (let ((spools cmail-mail-spools)
+	spool res server logname password options)
+    ;; cmail-mail-spools $B$,Dj5A$5$l$F$$$J$$>l9g!"B>$N>pJs$r$b$H$KF1$87A(B
+    ;; $B<0$N%j%9%H$r:n@.$7$F(B spools $B$KF~$l$k(B
+    (if (null spools)
+	(let (mf)
+	  (setq mf (getenv "MAIL"))
+	  (if (null mf)
+	      (progn
+		(setq mf cmail-spool-directory)
+		(or (string-match "^po:" mf)
+		    (setq mf (concat mf (user-login-name))))))
+	  (setq spools (list mf)))
+      (if (stringp spools)
+	  (setq spools (list spools))))
+    
+    ;; spools $B$N$=$l$>$l$NMWAG$K4X$7$F%k!<%W(B
+    (while spools
+      (setq spool (car spools))
+      (setq spools (cdr spools))
+      (if (and (listp spool) (memq 'skip spool))
+	  ;; $B%j%9%H$K(B 'skip $B$,F~$C$F$$$?>l9g$O=hM}$7$J$$(B
+	  nil
+	(cond
+	 ;; ("po:foo" "host" "option" "passwd") $B7A<0$N>l9g(B (POP)
+	 ((and (listp spool)
+	       (stringp (car spool))
+	       (string-match "\\`po:\\(.+\\)\\'" (car spool)))
+	  (setq logname (match-string 1 (car spool)))
+	  (setq server (or (cadr spool) ""))
+	  (setq options (or (caddr spool) ""))
+	  (setq password (cadddr spool))
+	  ;; if password is found in this list, remove and save it
+	  ;; somewhere else. don't leave it in this list.
+	  (if (and (stringp password) (not (string= password "")))
+	      (progn
+		(setcar (cdddr spool) nil)
+		(cmail-save-password 'pop3 server logname password)))
+	  (setq res (cons (list 'pop3 server logname password options) res)))
+	 
+	 ;; "po:user" $B$N7A<0$N>l9g(B (POP)
+	 ((and (stringp spool)
+	       (not (string-match "@" spool))
+	       (string-match "^po:\\(.+\\)$" spool))
+	  (setq logname (match-string 1 spool))
+	  (setq server (or (getenv "MAILHOST") ""))
+	  (setq password nil)
+	  (setq options (or (getenv "MOVEMAIL") ""))
+	  (if (string= server "")
+	      (cmail-error-resource 'get-spooled-mail-12)
+	    (setq res (cons (list 'pop3 server logname password options)
+			    res))))
+	 
+	 ;; "user@hostname" $B$N7A<0$N>l9g(B (POP)
+	 ((and (stringp spool)
+	       (or (featurep 'cmail-e19) (featurep 'cmail-20))
+	       (string-match "^\\(.*\\)@\\(.*\\)$" spool))
+	  (setq logname (match-string 1 spool))
+	  (setq server (match-string 2 spool))
+	  (setq password nil)
+	  (setq options (or (getenv "MOVEMAIL") ""))
+	  (if (string= server "")
+	      (cmail-error-resource 'get-spooled-mail-12)
+	    (setq res (cons (list 'pop3 server logname password options)
+			    res))))
+	 
+	 ;; "$B%9%W!<%k%U%!%$%kL>(B" $B$N>l9g(B
+	 ((stringp spool)
+	  (and (file-exists-p spool)
+	       (not (equal 0 (nth 7 (file-attributes spool))))
+	       (file-readable-p spool))
+	  (setq spool (expand-file-name spool))
+	  (setq res (cons (list 'local-file spool) res)))
+	 (t
+	  (cmail-error-resource 'get-spooled-mail-13)))))
+    (reverse res)))
+
+(defun cmail-prompt-for-password (server logname)
+  (let (hmesg pass)
+    (setq hmesg
+	  (cmail-format-resource1 'get-spooled-mail-4
+				  (concat logname "@" server)))
+    (require 'comint)
+    (setq pass (comint-read-noecho hmesg t))
+    (message "")
+    pass))
+
+(defun cmail-lookup-password (service server logname)
+  (let (password)
+    (setq password (cmail-account-db-get server service logname 'password))
+    (if (and (null password) (file-exists-p cmail-auth-file))
+	(progn
+	  (find-file cmail-auth-file)
+	  (setq buffer-read-only t)
+	  (goto-char (point-min))
+	  (if (re-search-forward
+	       (concat "^" server "\t+" (symbol-name service)
+		       "\t+" logname "\t+\\([^\t\n]*\\)$") nil t)
+	      (setq password (match-string 1)))
+	  (kill-buffer (current-buffer))
+	  (cmail-account-db-set server service logname 'password password)))
+    (and password (cmail-passwd-unscramble (copy-sequence password)))))
+
+(defvar *cmail-use-auth-file cmail-use-auth-file)
+(defun cmail-save-password (service server logname password)
+  (setq password (cmail-passwd-scramble (copy-sequence password)))
+  (if (and *cmail-use-auth-file
+	   (not (file-exists-p cmail-auth-file)))
+      (if (y-or-n-p (cmail-get-resource 'save-pass-1))
+	  (with-temp-buffer
+	    (let ((inhibit-quit t))
+	      (write-region (point-min) (point-max)
+			    (expand-file-name cmail-auth-file))
+	      (set-file-modes cmail-auth-file 384)
+	      (if (not (string-match
+			"\\`-r..------\\'"
+			(elt (file-attributes cmail-auth-file) 8)))
+		  (progn
+		    (delete-file cmail-auth-file)
+		    (cmail-message-resource 'save-pass-2)
+		    (setq *cmail-use-auth-file nil)))))
+	(setq *cmail-use-auth-file nil)))
+  (if (and *cmail-use-auth-file (file-exists-p cmail-auth-file))
+      (with-temp-buffer
+	(insert-file-contents cmail-auth-file)
+	(set-buffer-modified-p nil)
+	(goto-char (point-min))
+	(if (re-search-forward
+	     (concat "^" server "\t+" (symbol-name service)
+		     "\t+" logname "\t+\\([^\t\n]*\\)$") nil t)
+	    (if (not (string= password
+			      (buffer-substring (match-beginning 1)
+						(match-end 1))))
+		(progn
+		  (delete-region (match-beginning 1) (match-end 1))
+		  (goto-char (match-beginning 1))
+		  (insert password)))
+	  (if (y-or-n-p (cmail-get-resource 'save-pass-3))
+	      (progn
+		(goto-char (point-max))
+		(or (eobp) (insert "\n"))
+		(insert (concat server "\t" (symbol-name service) "\t"
+				logname "\t" password "\n")))))
+	(if (buffer-modified-p)
+	    (write-region (point-min) (point-max)
+			  (expand-file-name cmail-auth-file)))))
+  (cmail-account-db-set server service logname 'password password))
+
+(defconst *cmail-passwd-scramble-tab
+  (concat "q\\Wy10fvSm5r`LnE<?6lH_^4#&s[bewDQ%ZKo$Mh]\"tzjk X"
+	  "B:F),7VgT2Y8*.cd'}+=P@|iC{JGp3a-NAOIu;>~RU9!(/x"))
+
+(defun cmail-passwd-scramble (password)
+  (if (stringp password)
+      (let ((i 0) (len (length password)) j)
+	(while (< i len)
+	  (setq j (aref password i))
+	  (if (and (>= j 32) (< j 127))
+	      (aset password i (aref *cmail-passwd-scramble-tab (- j 32))))
+	  (setq i (1+ i)))))
+  password)
+
+(defun cmail-passwd-unscramble (password)
+  (if (stringp password)
+      (let ((i 0) (len (length password)) j n)
+	(while (< i len)
+	  (setq n (aref password i))
+	  (setq j 0)
+	  (while (and (< j 95)
+		      (not (= n (aref *cmail-passwd-scramble-tab j))))
+	    (setq j (1+ j)))
+	  (if (and (integerp j) (< j 95))
+	      (aset password i (+ 32 j)))
+	  (setq i (1+ i)))))
+  password)
+
+(defvar *cmail-account-db nil)
+
+(defun cmail-account-db-get (&rest key-list)
+  (cmail-account-db-get1 *cmail-account-db key-list))
+
+(defun cmail-account-db-get1 (alist key-list)
+  (if key-list
+      (let ((key-val (assoc (car key-list) alist)))
+	(if key-val (cmail-account-db-get1 (cdr key-val) (cdr key-list))))
+    alist))
+
+(defun cmail-account-db-set (key1 key2-val &rest key3-n-val)
+  (let* ((key-val-list (nreverse (cons key1 (cons key2-val key3-n-val))))
+	 (val (car key-val-list))
+	 (key-list (nreverse (cdr key-val-list))))
+    (if *cmail-account-db
+	(cmail-account-db-set1 *cmail-account-db key-list val)
+      (setq *cmail-account-db
+	    (list (cmail-account-db-make-list key-list val))))))
+
+(defun cmail-account-db-set1 (alist key-list val)
+  (let ((key-val (assoc (car key-list) alist)))
+    (if key-val
+	(if (cdr key-list)
+	    (cmail-account-db-set1 key-val (cdr key-list) val)
+	  (setcdr key-val val))
+      (nconc alist (list (cmail-account-db-make-list key-list val))))))
+
+(defun cmail-account-db-make-list (key-list val)
+  (if (cdr key-list)
+      (cons (car key-list)
+	    (list (cmail-account-db-make-list (cdr key-list) val)))
+    (cons (car key-list) val)))
+
 ;; ==== $B?7Ce%a%$%k$r%U%)%k%@$KDI2C(B ======================================
 (defun cmail-append-mail-to-folder (mailbuffer folder &optional summary)
   "MAILBUFFER$B$G;XDj$7$?%a%$%k$NFbMF$r(B, FOLDER$B$KDI2C$9$k(B."
@@ -501,7 +715,7 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
 $B=*$j$^$GFI$`$H<!$N%a%$%k$rI=<($9$k(B."
   (interactive (list (cmail-get-page-number-from-summary)))
   (let ((disp (get-buffer-window *cmail-mail-buffer)))
-    (if (= page 0)
+    (if (equal page 0)
 	(progn
 	  (setq *cmail-current-folder "")
 	  (setq *cmail-current-page 0)
@@ -510,7 +724,7 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
     (cmail-select-buffer *cmail-summary-buffer)
     (if (or (null disp)
 	    (not (string= cmail-current-folder *cmail-current-folder))
-	    (not (= page *cmail-current-page)))
+	    (not (equal page *cmail-current-page)))
 	  (cmail-show-contents page)
       (let* ((win (get-buffer-window *cmail-mail-buffer))
 	     (wh (window-height win))
@@ -1040,7 +1254,7 @@ interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
 	      (setq fldr (funcall (car al)))
 	      (if fldr (setq al nil)))
 	  (while (and (null fldr)
-		      (re-search-forward (format "^\\(%s\\): *"
+		      (re-search-forward (format "^\\(%s\\):\\s *"
 						 (car (car al))) max t))
 	    (let (beg end)
 	      (setq beg (point))
@@ -1081,15 +1295,24 @@ interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
 	(and buf (kill-buffer buf)))
       fldr)))
 
+(defvar *cmail-hist-mark-field-names nil)
 (defun cmail-mark-specified-mails (field value)
   "$B;XDj$7$?%U%#!<%k%I$NCM$K%^%C%A$9$k%a%$%k$K%^!<%/$r$D$1$k(B"
   (interactive
    (let* ((completion-ignore-case t)
 	  (hdr (completing-read (cmail-get-resource 'mark-specified-mails-1)
-				cmail-default-headers nil nil))
-	  (pat (read-string (cmail-format-resource1 'mark-specified-mails-2 hdr))))
+				cmail-default-headers nil nil nil
+				'*cmail-hist-mark-field-names))
+	  (hist (if (string= hdr "") (error (cmail-get-resource
+					     'mark-specified-mails-3))
+		  (intern (concat "*cmail-hist-mark-field-" (downcase hdr)))))
+	  (pat (progn
+		 (or (boundp hist) (set hist nil))
+		 (read-string (cmail-format-resource1
+			       'mark-specified-mails-2 hdr) nil hist))))
      (list hdr pat)))
   (let* ((found "found")
+	 (cmail-use-regexp-in-folder-alist t)
 	 (cmail-folder-alist (list (list field (cons value found))))
 	 (page 0) 
 	 (pos (cmail-get-page-number-from-summary)))
@@ -1525,3 +1748,48 @@ ECHO $B$,(Bnon-nil $B$N>l9g$O%;!<%V$7$J$+$C$?>l9g$K$=$N;]$rI=<($9$k(B."
   (mapcar 'cmail-dismiss-folder
 	  *cmail-opened-folders-list))
 ;; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+(defvar cmail-option-features '(all))
+(defun cmail-load-option-features ()
+  "cmail $B@lMQ$N(B optionnal feature $B$rFI$_9~$`!#(B"
+  (mapcar
+   (function
+    (lambda (dir)
+      (when (file-directory-p dir)
+	(let ((files (directory-files dir nil "\\`cmail-use-.*elc?\\'"))
+	      name)
+	  (mapcar
+	   (function
+	    (lambda (file)
+	      (string-match "\\`\\(cmail-use.*\\)\\.elc?\\'" file)
+	      (setq name (match-string 1 file))
+	      (when (or (memq 'all cmail-option-features)
+			(memq (intern-soft name) cmail-option-features))
+		(unless (featurep (intern name))
+		  ;; could not be provided unless intern
+		  (if (ignore-errors (require (intern name)))
+		      (message (cmail-format-resource 'options-load name)))))))
+	   files)))))
+   cmail-option-path-list))
+
+;;
+;; $B%*%W%7%g%sMQ%U%!%$%k$N%m!<%I$r:G8e$K$*$3$J$&!#(B
+;;
+(unless (and (fboundp 'cl-compiling-file) ; <- (require 'cl)
+	     (cl-compiling-file))
+; ... $B%3%s%Q%$%k;~$K$O<B9T$7$J$$!#(B
+
+  ;; ... $B%*%W%7%g%s%G%#%l%/%H%j$r(B load-path $B$KDI2C(B
+  (require 'path-util) ; <- add-path
+  (mapcar 'add-path cmail-option-path-list)
+
+  ;; ... $B%*%W%7%g%s%U%!%$%k$NAH$_9~$_(B
+  (cmail-load-option-features)
+
+  ;; ... custom $BMQ$N8@8l@_Dj(B
+  (cmail-i18n-swap-doc cmail-i18n-doc-language))
+
+;;; @ End.
+;;;
+
+;;; cmail.el ends here
